#ifndef __XIOS_CObjectFactory_impl__
#define __XIOS_CObjectFactory_impl__

#include "object_factory.hpp"

namespace xios
{
   /// ////////////////////// Définitions ////////////////////// ///
   template <typename U>
       int CObjectFactory::GetObjectNum(void)
   {
      if (CurrContext_ptr->size() == 0)
         ERROR("CObjectFactory::GetObjectNum(void)",
               << "please define current context id !");
      if(U::AllVectObj_ptr == NULL) return 0;
      return (*U::AllVectObj_ptr)[*CObjectFactory::CurrContext_ptr].size();
   }

   template <typename U>
      int CObjectFactory::GetObjectIdNum(void)
   {
      if (CurrContext_ptr->size() == 0)
         ERROR("CObjectFactory::GetObjectIdNum(void)",
               << "please define current context id !");
      if(U::AllMapObj_ptr == NULL) return 0;
      return (*U::AllMapObj_ptr)[*CObjectFactory::CurrContext_ptr].size();
   }

   template <typename U>
      bool CObjectFactory::HasObject(const StdString & id)
   {
      if (CurrContext_ptr->size() == 0)
         ERROR("CObjectFactory::HasObject(const StdString & id)",
               << "[ id = " << id << " ] please define current context id !");
      if(U::AllMapObj_ptr  == NULL)  return false;
      return ((*U::AllMapObj_ptr)[*CObjectFactory::CurrContext_ptr].find(id) !=
              (*U::AllMapObj_ptr)[*CObjectFactory::CurrContext_ptr].end());

   }

   template <typename U>
      bool CObjectFactory::HasObject(const StdString & context, const StdString & id)
   {
      if(U::AllMapObj_ptr  == NULL) return false;

      if (U::AllMapObj_ptr->find(context) == U::AllMapObj_ptr->end()) return false ;
      else return ((*U::AllMapObj_ptr)[context].find(id) !=  (*U::AllMapObj_ptr)[context].end());
   }

   template <typename U>
      std::shared_ptr<U> CObjectFactory::GetObject(const U * const object)
   {
      if(U::AllVectObj_ptr == NULL) return (std::shared_ptr<U>());
      if (CurrContext_ptr->size() == 0)
         ERROR("CObjectFactory::GetObject(const U * const object)",
               << "please define current context id !");
      std::vector<std::shared_ptr<U> > & vect = (*U::AllVectObj_ptr)[*CObjectFactory::CurrContext_ptr];

      typename std::vector<std::shared_ptr<U> >::const_iterator
         it = vect.begin(), end = vect.end();

      for (; it != end; it++)
      {
         std::shared_ptr<U> ptr = *it;
         if (ptr.get() == object)
            return (ptr);
      }

      ERROR("CObjectFactory::GetObject(const U * const object)",
               << "[type = " << U::GetName() << ", adress = " << object << "] "
               << "object was not found.");
      return (std::shared_ptr<U>()); // jamais atteint
   }

   template <typename U>
      std::shared_ptr<U> CObjectFactory::GetObject(const StdString & id)
   {
      if(U::AllMapObj_ptr  == NULL) return (std::shared_ptr<U>());
      if (CurrContext_ptr->size() == 0)
         ERROR("CObjectFactory::GetObject(const StdString & id)",
               << "[ id = " << id << " ] please define current context id !");
      if (!CObjectFactory::HasObject<U>(id))
         ERROR("CObjectFactory::GetObject(const StdString & id)",
               << "[ id = " << id << ", U = " << U::GetName() << " ] "
               << "object was not found.");
      return (*U::AllMapObj_ptr)[*CObjectFactory::CurrContext_ptr][id];
   }

   template <typename U>
      std::shared_ptr<U> CObjectFactory::GetObject(const StdString & context, const StdString & id)
   {
      if(U::AllMapObj_ptr  == NULL) return (std::shared_ptr<U>());

      if (!CObjectFactory::HasObject<U>(context,id))
         ERROR("CObjectFactory::GetObject(const StdString & id)",
               << "[ id = " << id << ", U = " << U::GetName() <<", context = "<<context<< " ] "
               << "object was not found.");
      return (*U::AllMapObj_ptr)[context][id];
   }

   template <typename U>
   std::shared_ptr<U> CObjectFactory::CreateObject(const StdString& id)
   {
      if(U::AllVectObj_ptr == NULL) U::AllVectObj_ptr = new xios_map<StdString, std::vector<std::shared_ptr<U> > >;
      if(U::AllMapObj_ptr  == NULL) U::AllMapObj_ptr  = new xios_map<StdString, xios_map<StdString, std::shared_ptr<U> > >;

      if (CurrContext_ptr->empty())
         ERROR("CObjectFactory::CreateObject(const StdString& id)",
               << "[ id = " << id << " ] please define current context id !");

      if (CObjectFactory::HasObject<U>(id))
      {
         return CObjectFactory::GetObject<U>(id);
      }
      else
      {
         std::shared_ptr<U> value(new U(id.empty() ? CObjectFactory::GenUId<U>() : id));

         (*U::AllVectObj_ptr)[*CObjectFactory::CurrContext_ptr].insert((*U::AllVectObj_ptr)[*CObjectFactory::CurrContext_ptr].end(), value);
         (*U::AllMapObj_ptr) [*CObjectFactory::CurrContext_ptr].insert(std::make_pair(value->getId(), value));

         return value;
      }
   }

   template <typename U>
   int CObjectFactory::CheckObjectVector()
   {
      if(U::AllVectObj_ptr)
      {
        return 1;
      }
      return 0;
   }
   

   template <typename U>
   const std::vector<std::shared_ptr<U> > &
         CObjectFactory::GetObjectVector(const StdString & context)
   {
      if(U::AllVectObj_ptr)
      {
        return (*U::AllVectObj_ptr)[context];
      }
      
   }

   template <typename U>
   const StdString& CObjectFactory::GetUIdBase(void)
   {
      static const StdString base = "__" + U::GetName() + "_undef_id_";
      return base;
   }

   template <typename U>
   StdString CObjectFactory::GenUId(void)
   {
      StdOStringStream oss;
      if(U::GenId_ptr == NULL) U::GenId_ptr = new xios_map< StdString, long int >;
      oss << GetUIdBase<U>() << (*U::GenId_ptr)[*CObjectFactory::CurrContext_ptr]++;
      return oss.str();
   }

   template <typename U>
   bool CObjectFactory::IsGenUId(const StdString& id)
   {
      const StdString& base = GetUIdBase<U>();
      return (id.size() > base.size() && id.compare(0, base.size(), base) == 0);
   }

} // namespace xios

#endif // __XIOS_CObjectFactory_impl__
